//
// Compile Method: 
//     csc /r:System.Runtime.Serialization.dll /r:System.ComponentModel.DataAnnotations.dll /r:..\..\tools\Json\Bin\DotNet\Newtonsoft.Json.dll jsontmplgen.cs
//
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.ComponentModel.DataAnnotations;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace com.vowei.Client
{
    public class JsonTemplateGen
    {
        public static void Main(string[] args)
        {
            if (args.Length != 1)
            {
                Trace.TraceError("Usage: JsonTemplateGen [model assembly]");
            }

            var assembly = Assembly.LoadFile(Path.GetFullPath(args[0]));
            var types = assembly.GetTypes();

            using (var stream = new StreamWriter("models.js"))
            {
                foreach (var type in types)
                {
                    if (!IsModelByConvention(type))
                    {
                        Trace.TraceWarning
                            (
                             string.Format(
                                   "Ignore invalid model {0}",
                                   type.FullName
                                   )
                             );
                        continue;
                    }

                    try
                    {
                        GenerateJsonDefinition(type, stream);
                        GenerateJQueryTmplDefinition(type);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("Error process type {0} with excetpion {1}", type.FullName, e);
                    }
                }
            }
        }

        private static void GenerateJQueryTmplDefinition(Type type)
        {
            Debug.Assert(type != null);

            var file = string.Format("{0}.tmpl", type.Name);
            using (var stream = new StreamWriter(file))
            {
                GenerateJQueryTmplDefinitionImpl(type, stream);
            }
        }

        private static List<PropertyInfo> GetOrderedProperties(Type type)
        {
            var result = new List<PropertyInfo>();
            var typeHierarchy = new Stack<Type>();
            typeHierarchy.Push(type);

            while (type.BaseType != null)
            {
                typeHierarchy.Push(type.BaseType);
                type = type.BaseType;
            }

            while (typeHierarchy.Count > 0)
            {
                var iter = typeHierarchy.Pop();
                result.AddRange(iter.GetProperties(BindingFlags.DeclaredOnly | BindingFlags.Instance | BindingFlags.Public));
            }

            // Sort the properties order by Requirement
            result.Sort(PropertyComparison);

            return result;
        }

        private static int PropertyComparison(PropertyInfo left, PropertyInfo right)
        {
            var attrLeft = left.GetCustomAttributes(typeof(RequiredAttribute), false);
            var attrRight = right.GetCustomAttributes(typeof(RequiredAttribute), false);

            var leftHas = attrLeft != null && attrLeft.Length > 0;
            var rightHas = attrRight != null && attrRight.Length > 0;

            if (leftHas && rightHas)
                return 0;
            else if (leftHas && !rightHas)
                return -1;
            else if (!leftHas && rightHas)
                return 1;
            else
                return 0;
        }

        private static void
            GenerateJQueryTmplDefinitionImpl(Type type,
                             StreamWriter stream)
        {
            var properties = GetOrderedProperties(type);
            var attr = type.GetCustomAttributes(typeof(DisplayAttribute), false);

            stream.WriteLine("<div>");
            stream.WriteLine("  <fieldset>");
            if (attr != null && attr.Length > 0)
            {
                stream.WriteLine(string.Format("<legend>{0}</legend>",
                                   ((DisplayAttribute)attr[0]).Name));
            }

            foreach (var property in properties)
            {
                if (property.Name == "Id")
                    continue;

                attr = property.GetCustomAttributes
                    (typeof(DisplayAttribute), false);

                stream.WriteLine("    <div class='editor-label'>");
                if (attr != null && attr.Length > 0)
                {
                    stream.WriteLine(((DisplayAttribute)attr[0]).Name);
                }
                else
                {
                    stream.WriteLine(property.Name);
                }

                stream.WriteLine("    </div>");

                stream.WriteLine("    <div class='editor-field'>");
                if (property.CanWrite)
                {
                    var propertyType = property.PropertyType;
                    if (propertyType.IsGenericType &&
                     propertyType.GetGenericTypeDefinition() == typeof(Nullable<>))
                        propertyType = propertyType.GetGenericArguments()[0];

                    stream.WriteLine(string.Format("      <input type='text' id='{0}' value='${{{0}}}' class='{1}Class' />",
                                   property.Name,
                                   propertyType.Name
                                   )
                             );

                    attr = property.GetCustomAttributes(
                        typeof(RequiredAttribute), false);
                    if (attr != null && attr.Length > 0)
                    {
                        stream.WriteLine("      <span>*</span>");
                    }
                }

                stream.WriteLine("    </div>");
            }

            stream.WriteLine("    <input type='button' value='ύ' class='edit-field' />");
            stream.WriteLine("  </fieldset>");
            stream.WriteLine("</div>");
        }

        private static bool IsModelByConvention(Type type)
        {
            Debug.Assert(type != null);

            bool result = type.FullName.IndexOf(".Models.") > 0 &&
            type.GetConstructor(new Type[] { }) != null;

            if (type.BaseType != null && type.BaseType.Name == "DbContext")
                result = false;

            return result;
        }

        private static void GenerateJsonDefinition(Type type,
                               StreamWriter stream)
        {
            Debug.Assert(type != null);
            Debug.Assert(type != null);

            var ctor = type.GetConstructor(new Type[] { });
            var obj = ctor.Invoke(new object[] { });

            stream.Write(string.Format("var {0} = ", type.Name.ToLower()));
            stream.Write(JsonConvert.SerializeObject(obj, new JavaScriptDateTimeConverter()));
            //var ser = new DataContractJsonSerializer(type);
            //ser.WriteObject(stream.BaseStream, obj);
            stream.WriteLine(";");
        }
    }
}